(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: expor.mli,v 1.15 2014/01/21 09:21:15 fclement Exp $ *)

(* Exportable closures handling *)

(* Closures that can be exported are the result of lambda-lifted [seq]
  enclosed expressions, two things may happen, when we lift such an
  expression, if it has free parameters, we will lift with the help of
  a tuple in order to keep a unified arity :
    [fun (a,b,c) -> EXPR (a,b,c)]
  On the contrary, if no parameters are free, we lift using a tuple,
  again to keep the same arity :
    [fun () -> EXPR]
*)

(* Thus, lifted [seq] enclosed expressions have the following type :
    [something -> 'a -> 'b]
  Generally ['a] and ['b] are streams.
*)

type ('x, 'a, 'b) llexpr;;

val register :
    ('x -> 'a Sklstream.t -> 'b Sklstream.t) -> ('x, 'a, 'b) llexpr
;;
val retreive :
    ('x, 'a, 'b) llexpr -> ('x -> 'a Sklstream.t -> 'b Sklstream.t)
;;
val unsafe_retreive : int -> ('x -> 'a Sklstream.t -> 'b Sklstream.t);;
val get_exportable : ('x * ('x, 'a, 'b) llexpr) -> (Obj.t * int);;
val is_registered :
  ('x -> 'a Sklstream.t -> 'b Sklstream.t) -> ('x, 'a, 'b) llexpr option
;;
