(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parfun.ml,v 1.21 2013/03/29 13:21:41 fclement Exp $ *)

let rec parfun
: 'a 'b. ('a, 'b) Skl.t -> 'a Sklstream.t -> 'b Sklstream.t =
  fun sk ->
    match sk with
    | Sk.Skl.Par par ->
      Sk.with_par par {
        Sk.f_par = fun skl1 skl2 ->
          let f1 = parfun skl1 in
          let f2 = parfun skl2 in
          fun x -> Sklstream.in_par (f1, f2) x
      }
    | Sk.Skl.Rails rails ->
      Sk.with_rails rails {
        Sk.f_rails = fun ska ->
          let fv = Array.map parfun ska in
          fun x  -> Sklstream.on_rails fv x
      }
    | Sk.Skl.Sum sum ->
      Sk.with_sum sum {
        Sk.f_sum = fun coerce_in skl1 skl2 ->
          let f1 = parfun skl1 in
          let f2 = parfun skl2 in
          fun x ->
            Sklstream.in_sum (f1, f2) (Sklstream.map coerce_in x)
      }
    | Sk.Skl.Pipe pipe ->
      Sk.with_pipe pipe {
        Sk.f_pipe = fun skl1 skl2 ->
          let f1 = parfun skl1 in
          let f2 = parfun skl2 in
          fun x -> f2 (f1 x)
      }
    | Sk.Skl.Atom atom ->
      Sk.with_atom atom {
        Sk.f_atom = fun params llexp _ ->
          let f_gen = (Expor.retreive llexp) in
          let f = f_gen params in
          fun x -> f x
      }
    | Sk.Skl.Id id_skl ->
      Sk.with_id id_skl {
        Sk.f_id = fun f ->
          Sklstream.map f
      }
    | Sk.Skl.Prod prod ->
      Sk.with_prod prod {
        Sk.f_prod = fun coerce_in coerce_out skl1 skl2 ->
          let f1 = parfun skl1 in
          let f2 = parfun skl2 in
          fun x ->
            Sklstream.map coerce_out (
              Sklstream.in_prod (f1, f2) (Sklstream.map coerce_in x))
      }
    | Sk.Skl.Loop loop ->
      Sk.with_loop loop {
        Sk.f_loop = fun coerce pskl skl ->
          let p = parfun pskl in
          let f = parfun skl in
          fun x -> Sklstream.map coerce (Sklstream.loop p f x)
      }
;;

type parfun =
  { parfun : 'a 'b. ('a, 'b) Skl.skel -> 'a Sklstream.t -> 'b Sklstream.t; }
;;

let pardo f =
  f { parfun = parfun; }
;;
(** Entry point *)
