(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: batch.mli,v 1.11 2014/01/21 09:21:17 fclement Exp $ *)

type batch;;

type fail_action =
  | Fail_error of string
  | Fail_warn of string
  | Fail_ignore
;;

type task_type =
  | Background
  | Normal
  | Timeouted of int
;;

type shell_action = {
  program : string;
  args : string array;
};;

type pid = int;;

type batch_error =
  | Batch_timeout of shell_action
  | Batch_shell_error of (int * shell_action)
  | Batch_failure
;;

type batch_run_return =
  | Ok of pid list
  | Error of (pid list * batch_error)
;;

val empty : batch;;

val add_shell_task :
  batch -> fail_action -> shell_action -> task_type -> batch;;
val add_subscript_task : batch -> fail_action -> batch -> batch;;
val add_pipe_task :
  batch -> fail_action -> batch -> (in_channel -> batch) -> batch;;
val add_control_task :
  batch -> fail_action -> (pid list -> batch) -> batch;;
val add_echo_task : batch -> string -> batch;;
val add_fail_task : batch -> string -> batch;;

val eval_batch : batch -> out_channel -> out_channel -> batch_run_return;;
